﻿using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.EvolutionTracking;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  public class SymbolicDataAnalysisExpressionAfterManipulatorOperator : BeforeManipulatorOperator<ISymbolicExpressionTree> {
    private readonly SymbolicExpressionTreeNodeSimilarityComparer comparer;

    public SymbolicDataAnalysisExpressionAfterManipulatorOperator() {
      comparer = new SymbolicExpressionTreeNodeSimilarityComparer {
        MatchVariableNames = true,
        MatchVariableWeights = true,
        MatchConstantValues = true
      };
    }

    public override IOperation Apply() {
      var vChild = (IGenealogyGraphNode<ISymbolicExpressionTree>)GenealogyGraph[ChildParameter.ActualValue].First();
      var nodesBefore = (List<ISymbolicExpressionTreeNode>)vChild.InArcs.First().Data;
      var nodesAfter = ChildParameter.ActualValue.IterateNodesBreadth().ToList();
      IFragment<ISymbolicExpressionTreeNode> fragment = null;

      for (int i = 0; i < Math.Min(nodesAfter.Count, nodesBefore.Count); ++i) {
        if (comparer.Equals(nodesAfter[i], nodesBefore[i])) continue;
        fragment = new Fragment<ISymbolicExpressionTreeNode> {
          Root = nodesAfter[i],
          Index = i
        };
      }

      vChild.InArcs.First().Data = fragment;
      return base.Apply();
    }
  }
}
